import pytest
from unittest import TestCase
import os
from acme_news.engines.google_news import GoogleNewsEngine
from typing import List, Dict

class TestGoogleNews(TestCase):
    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_query(self):
        gnews = GoogleNewsEngine(search_query='iraq')
        results = gnews.parse_google_news_links()
        return self.assertEqual(100, len(results))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_parse_content(self):
        gnews = GoogleNewsEngine()
        results = gnews.parse_news_contents(url='https://news.google.com/articles/CAIiEDZCK0jVl3DOy5QS58lIgX0qGQgEKhAIACoHCAow4OH9CjCg6_UCMJ-42gU?hl=en-US&gl=US&ceid=US%3Aen')
        return self.assertEqual(2, len(results))



#!/usr/bin/env python
from acme_news.browser.acme_browser import ACMEBrowser
from selenium.webdriver.phantomjs.webdriver import WebDriver
from typing import List, Dict
from base64 import urlsafe_b64encode
import re
class RussiaToday(ACMEBrowser):

    def __init__(self, query: str = "",
                 category: str = "politics",
                 content_type: str = "News"):
        """
        DESCRIPTION
            Helper class to collect the contents from Russia Today

        PARAMETERS
            :param query:
            :param category:
            :param content_type:

        FUNCTIONS
            __init__
            __set_extra_params__
            __get_extra_params__
            __build_search_query__
            parse_page_content
            parse_video_link
            get_video_element
            parse_rt_contents
            rt_bulk_quries
        """

        self.rt_url: str = "https://www.rt.com/search"
        self.extra_query_params: str = ""
        self.rt_link: str = ""
        self.rt_driver = None
        self.query: str = query
        self.category: str = category
        self.content_type: str = content_type
        self.__build_search_query__()
        super(RussiaToday, self).__init__()

    def __set_extra_params__(self, page_size: int, page: int):
        if not all([page, page_size]) and (page <= 0 and page_size <= 0):
            raise ValueError("Please provide the right values for page and the page_size")

        self.extra_query_params = f"&pageSize={page_size}&format=&page={page}"

    def __get_extra_params__(self):
        return f"{self.search_query}{self.extra_query_params}"

    def __build_search_query__(self):
        self.search_query = f"{self.rt_url}?q={self.query}&type={self.content_type}&xcategory={self.category}"

    def parse_page_content(self, url: str) -> Dict:
        if not url and not re.match(r"^https://www.rt.com", url):
            raise ValueError("Please provide the correct URLs.")

        self.rt_driver = self.browser(url=url)
        return {
            'title': self.rt_driver.title,
            'url': self.rt_driver.current_url,
            'category': self.category,
            'content': '\n'.join([tag.text for tag in self.rt_driver.find_elements_by_tag_name('p') if not tag.get_attribute('class')]),
            'image_urls': urlsafe_b64encode(str([tag.get_attribute('src') for tag in self.rt_driver.find_elements_by_tag_name('img') if 'cdni' in str(tag.get_attribute('src'))]).encode('utf-8')),
            'date': self.rt_driver.find_element_by_tag_name('body').find_element_by_css_selector('span.date.date_article-header').text
        }

    def parse_video_link(self, url: str) -> Dict:
        if not url and not re.match(r"^https://www.rt.com", url):
            raise ValueError("Please provide the correct URLs.")

        self.rt_driver = self.browser(url=url)

        return {
            'title': self.rt_driver.title,
            'url': self.rt_driver.current_url,
            'video_url': self.get_video_element(rt_driver=self.rt_driver, content_type='video'),
            'image_urls': self.get_video_element(rt_driver=self.rt_driver, content_type='image'),
            'date': self.rt_driver.find_element_by_tag_name('body').find_element_by_css_selector('span.date.date_article-header').text,
            'content': self.rt_driver.find_element_by_tag_name('body').text
        }

    def get_video_element(self, rt_driver: WebDriver, content_type: str) -> str:
        try:
            return list(set([x.get_attribute("content") for x in rt_driver.find_element_by_tag_name('head').find_elements_by_tag_name('meta') if content_type in str(x.get_attribute("property"))]))[0]

        except:
            return "NONE"

    def parse_rt_contents(self) -> List[Dict]:
        results: List = []
        self.rt_driver = self.browser(url=self.search_query)
        rt_response = self.rt_driver.find_element_by_class_name("news-block").find_element_by_class_name('card-list').find_elements_by_css_selector('li.card-list__item')

        for element in rt_response:
            try:
                summary = element.find_element_by_css_selector('div.card__summary')
                current_article: Dict = {
                    'region_name': self.query,
                    'url': summary.find_element_by_css_selector('a.link').get_attribute('href'),
                    'title': element.find_element_by_css_selector('strong.card__header.card__header_serp-list').text,
                    'news_date': element.find_element_by_css_selector('time.date').text,
                    'category': self.category,
                    'summary': summary.text
                }
                results.append(current_article)
            except:
                pass

        self.rt_driver.close()
        return results

    def rt_bulk_quries(self, page: int, page_size: int) -> List[Dict]:
        self.__set_extra_params__(page_size=page_size, page=page)

        results: List = []
        self.rt_driver = self.browser(url=self.__get_extra_params__())

        rt_response = self.rt_driver.find_elements_by_class_name("news-block")
        if not rt_response:
            raise ConnectionError(f"Unable to parse the contents from RT {self.__get_extra_params__()}")

        for content in rt_response:
            try:
                contents = content.find_element_by_class_name('card-list').find_elements_by_css_selector('li.card-list__item')
                for element in contents:
                    card = element.find_element_by_css_selector('div.card')

                    contents: Dict = {
                        'region_name': self.query,
                        'url': card.find_element_by_css_selector('a.link.link_hover').get_attribute('href'),
                        'title': card.find_element_by_css_selector('a.link.link_hover').text,
                        'category': self.category,
                        'content': card.text,
                        'image_url': card.find_element_by_css_selector('img.media__item.media__item_ratio').get_attribute('src'),
                        'date': card.find_element_by_css_selector('time.date').text
                    }
                    results.append(contents)
            except:
                pass

        self.rt_driver.close()
        return results



import pytest
from unittest import TestCase
import os
from acme_news.engines.russia_today import RussiaToday
from typing import List, Dict

class TestRussiaToday(TestCase):

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_rt_parse_content(self):
        rt = RussiaToday(query='Iraq')
        results: List[Dict] = rt.parse_rt_contents()
        return self.assertEqual(10, len(results))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_rt_bulk_quries(self):
        rt = RussiaToday(query='Iraq', content_type='Telecast')
        results: List[Dict] = rt.rt_bulk_quries(page=2, page_size=10)
        return self.assertEqual(10, len(results ))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_rt_page_content(self):
        rt = RussiaToday()
        results: Dict = rt.parse_page_content(url="https://www.rt.com/usa/519626-msnbc-joe-vaccine-passports/")
        expected_result: str = "Support vaccine passports or go back to your caves, 'idiots,' MSNBC’s Joe Scarborough tells Republicans and civil liberties fans — RT USA News"
        return self.assertEqual(expected_result, results.get('title'))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_rt_parse_video_link(self):
        rt = RussiaToday()
        results: Dict = rt.parse_video_link(url="https://www.rt.com/shows/sputnik/518599-china-west-iraq-pope/")
        expected_result: str = "The Far East and the Middle East (E376) — RT Sputnik Orbiting the WorldAA"
        return self.assertEqual(expected_result, results.get('title'))



import pytest
from unittest import TestCase
import os
from acme_news.engines.cnn import CNNEngine
from typing import List, Dict

class TestBrowser(TestCase):

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_query(self):
        cnn = CNNEngine(news_article='iraq')
        self.assertEqual("https://www.cnn.com/search?size=10&q=iraq&category=politics", cnn.get_query())

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_all_queries(self):
        cnn = CNNEngine(news_article='iraq', page_depth=10)
        quries: List[str] = cnn.get_all_queries()
        self.assertEqual(1, len(quries))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_browser(self):
        cnn = CNNEngine(url="https://www.cnn.com/2021/03/19/world/meanwhile-in-america-march-19-intl-latam/index.html")
        return self.assertEqual(cnn.get_article_title(), "The bias facing Black and Asian Americans reflects a broader problem - CNN")


    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_cnn_links(self):
        cnn = CNNEngine(news_article='iraq')
        cnn_links: List[str] = cnn.get_links()
        return self.assertEqual(10, len(cnn_links))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_bulk_cnn_links(self):
        cnn = CNNEngine(news_article='iraq', page_depth=10)
        cnn_links: List[str] = cnn.get_all_links()
        return self.assertEqual(10, len(cnn_links))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_cnn_api_request(self):
        cnn = CNNEngine(news_article='iraq', category='all', api_endpoint=True)
        cnn_links: List[Dict] = cnn.get_page_content()
        return self.assertEqual(10, len(cnn_links))


    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_all_contents(self):
        cnn = CNNEngine(news_article='iraq', category='politics', api_endpoint=True, page_depth=10)
        contents: List[Dict] = cnn.get_all_contents()
        return self.assertEqual(10, len(contents))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_video_links(self):
        cnn = CNNEngine(news_article='iraq',
                        category='politics',
                        api_endpoint=True,
                        video=True)
        contents: List[Dict] = cnn.get_video_links()
        return self.assertEqual(10, len(contents))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_all_video_links(self):
        cnn = CNNEngine(news_article='iraq',
                        category='politics',
                        api_endpoint=True,
                        page_depth=10,
                        video=True)
        contents: List[Dict] = cnn.get_all_video_links()
        return self.assertEqual(10, len(contents))



#!/usr/bin/env python
import os
import re
import gzip
from typing import List, Dict
from uuid import uuid4

from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow

from nltk import sent_tokenize
from nltk.sentiment.vader import SentimentIntensityAnalyzer
import pandas as pd

def tarball(file_name: str, output_path: str = "") -> Dict:
    if not output_path:
        output_path: str = re.sub(r"^./", "", file_name).rstrip("/").replace("/", "_")

    tar_ball: str = f"{output_path}_news_middle_east_regions_{str(uuid4())}.tar.gz"

    joined_contents = None
    current_content = None

    if os.path.isdir(file_name):
        contents: List = []
        for root, dirs, content_files in os.walk(file_name):
            for csv_file in content_files:
                current_file_path: str = os.path.join(root, csv_file)
                if os.path.getsize(current_file_path) > 0:
                    contents.append(open(current_file_path, 'r').read())

        joined_contents = '\n'.join(contents).encode('utf-8')

    if os.path.isfile(file_name):
        try:
            current_content = open(file_name, 'r').read().encode('utf-8')

        except IOError as e:
            raise IOError(f"Unable to open {file_name}") from e

    with gzip.open(tar_ball, 'w') as f:
        f.write(joined_contents if joined_contents else current_content)
    f.close()

    return {'file': tar_ball,
            'status': "created" if os.path.exists(tar_ball) else "failed"
           }


def upload_to_drive(cred_file: str,
                   file_name: str,
                   api_scope: str = "https://www.googleapis.com/auth/drive",
                   mimetype: str = 'application/gzip') -> Dict:

    if not os.path.exists(cred_file):
        raise FileNotFoundError(f"Unable to find the following file {cred_file}")

    if not os.path.exists(file_name):
        raise FileNotFoundError(f"Unable to find the following file {file_name}")

    creds = None
    try:
        creds = Credentials.from_authorized_user_file(cred_file)
    except:
        pass

    if not creds and not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())

        else:
            flow = InstalledAppFlow.from_client_secrets_file(cred_file, api_scope)
            creds = flow.run_local_server(port=58887)

        with open(os.path.join(os.path.expanduser("~/"), cred_file)) as f:
            f.write(creds.to_json())

    google_drive = build('drive', 'v3', credentials=creds)
    media = MediaFileUpload(file_name, mimetype=mimetype, resumable=True)
    file_metadata: Dict = {'name': file_name,
                           'mimeType': 'application/gzip'
                           }
    response = google_drive.files().create(body=file_metadata, media_body=media, fields='id').execute()

    return response

def get_sentiment_scores(news_article: str) -> Dict:
    return pd.DataFrame([SentimentIntensityAnalyzer().polarity_scores(sentence) for sentence in sent_tokenize(news_article)]).mean().to_dict()


#!/usr/bin/envp python
from acme_news.browser.acme_browser import ACMEBrowser
from typing import List, Dict

class GMANEngine(ACMEBrowser):

    def __init__(self):
        self.search_query: str = ""
        self.page_size: int = 0
        self.page: int = 0
        self.gman_headers: Dict = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:87.0) Gecko/20100101 Firefox/87.0',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
            'Accept-Language': 'en-US,en;q=0.5',
            'DNT': '1',
            'Connection': 'keep-alive',
            'Upgrade-Insecure-Requests': '1',
            'Sec-GPC': '1'
        }
        super(ACMEBrowser).__init__()

    def get_pagesize(self) -> int:
        return self.page_size

    def get_page(self) -> int:
        return self.page

    def get_search_quer(self) -> str:
        return self.search_query

    def get_gman_reports(self, search_query: str, page: int = 1, page_size: int = 20, sort: str = "descending") -> List[Dict]:

        self.search_query = search_query.replace(" ", "+")
        self.page = page
        self.page_size = page_size

        if not self.search_query:
            raise ValueError("Please provide the right search query.")

        if not all([self.page, self.page_size]) and self.page <= 0 and self.page_size <= 0:
            raise ValueError("Please provide the right page and page_size")

        self.search_query: str = f"https://s.fbi.gov/?SearchableText={self.search_query}&searchHelpText=To+narrow+your+search%2C+select+a+content+type+option+listed+under+%E2%80%9CMore.%E2%80%9D+To+broaden+your+search+to+other+FBI+sites%2C+select+a+subdomain+listed+under+%E2%80%9CSource.%E2%80%9D&pageSize={self.page_size}&page={self.page}&sort_on=&sort_order={sort}&after="

        response: List[Dict] = self.transform_requests(response=self.fetch_api_request(api_endpoint=self.search_query, headers=self.gman_headers)).get('results', {})

        return self.normalize(response=response)




import pytest
from unittest import TestCase
import os
from acme_news.engines.gman import GMANEngine
from typing import Dict, List

class TestBrowser(TestCase):

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_query(self):
        gman = GMANEngine()
        results: List[Dict] = gman.get_gman_reports(search_query="iran", page_size=10)
        self.assertEqual(10, len(results))
